import React, { Component, createRef } from 'react';
import PropTypes from 'prop-types';
import MUIMenu from '@material-ui/core/Menu';
import { createMuiTheme, MuiThemeProvider } from '@material-ui/core/styles';

// Styles
import getStyles from './Menu.styles';

/**
 * A Menu displays a list of choices on a temporary surface. It appears when the user interacts with a button, or other control.
 * @see https://material-ui.com/components/menus/
 */
class Menu extends Component {
    constructor(props) {
        super(props);
        this.menuRef = createRef();
        this.state = {
            anchorEl: null,
        };
    }

    render() {
        const {
            button,
            children,
            contextStyles,
            testID,
        } = this.props;

        const {
            anchorEl,
        } = this.state;

        // Inject the ability to close the menu on item click
        const kids = React.Children.map(children, (child) => {
            const {
                children: itemChildren,
                onClick,
            } = child.props;
            return React.cloneElement(
                child,
                {
                    onClick: (...args) => {
                        if (onClick) {
                            onClick(...args);
                        }
                        this.handleClose();
                    },
                },
                itemChildren,
            );
        });

        // Inject the ability to open the menu on button click
        const {
            children: buttonChildren,
            onClick: buttonClick,
        } = button.props;
        const Trigger = React.cloneElement(
            button,
            {
                onClick: (...args) => {
                    this.handleClick(...args);
                    if (buttonClick) {
                        buttonClick(...args);
                    }
                },
            },
            buttonChildren,
        );

        const styles = getStyles(contextStyles);
        const theme = this.getTheme(styles);

        return (
            <div
                ref={ this.menuRef }
                style={ styles.container }
                data-testid={ testID && `${testID}__menuContainer` }>
                { Trigger }
                <MuiThemeProvider theme={ theme }>
                    <MUIMenu
                        id="long-menu"
                        anchorEl={ anchorEl }
                        keepMounted
                        open={ !!anchorEl }
                        onClose={ this.handleClose }
                        data-testid={ testID && `${testID}__menu` }>
                        { kids }
                    </MUIMenu>
                </MuiThemeProvider>
            </div>
        );
    }

    /**
     * Helper method to set the anchor element on button click
     * @param {Object} event - click event of button
     */
    handleClick = (event) => {
        this.setState({
            anchorEl: event.currentTarget,
        });
    };

    /**
     * Helper method to close the anchor element
     */
    handleClose = () => {
        this.setState({
            anchorEl: null,
        });
    };

    /**
     * Helper method to get the theme
     * @param {Object} styles - Styles that are used to create the theme
     */
    getTheme = (styles) => createMuiTheme({
        overrides: {
            MuiMenu: {
                paper: {
                    ...styles.menu,
                },
            },
        },
        typography: {
            useNextVariants: true,
        },
    });

    /**
     * Helper method to set the anchor element. This is used in order to set autoFocus
     * @param {Object} event - The anchorl element target
     */
    show = () => {
        this.setState({
            anchorEl: this.menuRef.current,
        });
    };
}

Menu.propTypes = {

    /** Button element that is used for triggering Menu */
    button: PropTypes.node.isRequired,

    /** Menu Items that needs to be displayed inside the Menu */
    children: PropTypes.arrayOf(PropTypes.node).isRequired,

    /** Styles for this component */
    contextStyles: PropTypes.shape({
        /** Styles for the container surrounding menu and button */
        container: PropTypes.object,

        /** Styles for the menu container */
        menu: PropTypes.object,
    }),

    /** Id used for testing */
    testID: PropTypes.string,
};

Menu.defaultProps = {
    contextStyles: {},
};

export default Menu;
